function T = dwhtmtx(n)
%   Discrete Walsh-Hadamard transform matrix.
%   n is an integer scalar of class double.
%   T is returned as a n x n matrix of class double.
%   
narginchk(1,1);
validateattributes(n,{'double'},{'integer' 'scalar'});
h = hadamard(n);
HadIdx = 0:n-1;                            % Hadamard index
M = log2(n)+1;                             % Number of bits to represent the index
binHadIdx = fliplr(dec2bin(HadIdx,M))-'0'; % Bit reversing of the binary index
binSeqIdx = zeros(n,M-1);                  % Pre-allocate memory
for k = M:-1:2
    binSeqIdx(:,k) = xor(binHadIdx(:,k),binHadIdx(:,k-1));
end
SeqIdx = binSeqIdx*pow2((M-1:-1:0)');    % Binary to integer sequency index
T = h(SeqIdx+1,:)/sqrt(n);               % 1-based indexing
