function y = ifwht2(x)
%   2-d discrete Walsh-Hadamard transform.
%   x is a 2-d matrix of class double.
%   The size must be a power of 2.
%   y is returned as a matrix of class double.
%   
narginchk(1,1);
validateattributes(x,{'double'},{'2d'});
[m, n] = size(x);
if bitand(uint64(m),uint64(m-1)) ~= uint64(0)
    error('the number of rows is not power of 2');
elseif bitand(uint64(n),uint64(n-1)) ~= uint64(0)
    error('the number of columns is not power of 2');
end
y = zeros(m,n);
for k=1:n
    y(:,k) = ifwht(x(:,k));
end
for k=1:m
    y(k,:) = ifwht(y(k,:));
end